@@ -6,6 +6,12 @@ from redis.connection import (
     SSLConnection,
     UnixDomainSocketConnection
 )
+from redis.sentinel import (
+    Sentinel,
+    SentinelConnectionPool,
+    SentinelManagedConnection,
+    SentinelManagedSSLConnection,
+)
 from redis.utils import from_url
 from redis.exceptions import (
     AuthenticationError,
@@ -31,7 +37,9 @@ def int_or_str(value):
         return value
 
 
-__version__ = '3.5.3'
+__version__ = "4.0.0"
+
+
 VERSION = tuple(map(int_or_str, __version__.split('.')))
 
 __all__ = [
@@ -51,6 +59,10 @@ __all__ = [
     'Redis',
     'RedisError',
     'ResponseError',
+    'Sentinel',
+    'SentinelConnectionPool',
+    'SentinelManagedConnection',
+    'SentinelManagedSSLConnection',
     'SSLConnection',
     'StrictRedis',
     'TimeoutError',
